import { Notice, Setting } from 'obsidian'
import type { LocatorSettings } from './utils'
import { isCacheEnabled } from './utils'
import { saveSettings } from './utils'
import { htmlDescription, isPluginDisabled, needsARestart } from './utils'
import type LocatorPlugin from 'src/main'
import { K_DISABLE_OMNISEARCH } from 'src/globals'

export function injectSettingsDanger(
  plugin: LocatorPlugin,
  settings: LocatorSettings,
  containerEl: HTMLElement
) {
  const database = plugin.database

  new Setting(containerEl).setName('Danger Zone').setHeading()

  // Disable Locator
  const disableDesc = new DocumentFragment()
  disableDesc.createSpan({}, span => {
    span.innerHTML = `Disable Locator on this device only.<br>
      ${needsARestart}`
  })
  new Setting(containerEl)
    .setName('Disable on this device')
    .setDesc(disableDesc)
    .addToggle(toggle =>
      toggle.setValue(isPluginDisabled(plugin.app)).onChange(async v => {
        if (v) {
          plugin.app.saveLocalStorage(K_DISABLE_OMNISEARCH, '1')
          new Notice('Locator - Disabled. Please restart Obsidian.')
        } else {
          plugin.app.saveLocalStorage(K_DISABLE_OMNISEARCH) // No value = unset
          new Notice('Locator - Enabled. Please restart Obsidian.')
        }
      })
    )

  // Force save cache
  new Setting(containerEl)
    .setName('Force save the cache')
    .setDesc(
      htmlDescription(`Locator has a security feature that automatically disables cache writing if it cannot fully perform the operation.<br>
      Use this option to force the cache to be saved, even if it causes a crash.<br>
      ⚠️ <span style="color: var(--text-accent)">Enabling this setting could lead to crash loops</span>`)
    )
    .addToggle(toggle =>
      toggle.setValue(settings.DANGER_forceSaveCache).onChange(async v => {
        settings.DANGER_forceSaveCache = v
        await saveSettings(plugin)
      })
    )

  // Clear cache data
  if (isCacheEnabled()) {
    new Setting(containerEl)
      .setName('Clear cache data')
      .setDesc(
        htmlDescription(`Erase all Locator cache data.
        Use this if Locator results are inconsistent, missing, or appear outdated.<br>
        ${needsARestart}`)
      )
      .addButton(btn => {
        btn.setButtonText('Clear cache')
        btn.onClick(async () => {
          await database.clearCache()
        })
      })
  }
}
